# http://www.igvita.com/blog (Ilya Grigorik)

module Canvas
  class CalendarView < Canvas
    def initialize(name, date)
      super(name)
      
      @topRow = 50
      @rowH = (@doc.pdf.page_height.to_i - @topRow) / 6
      @columnW = (@doc.pdf.page_width.to_i) / 7
      @doc.pdf.stroke_color! Color::RGB.new(140,140,140)
      
      # Draw calendar grid
      1.upto(6) do |n|
        @doc.pdf.line(@doc.pdf.absolute_left_margin, n*@rowH, @doc.pdf.absolute_right_margin, n*@rowH).stroke
      end
      
      1.upto(6) do |n|
        @doc.pdf.line(n*@columnW, @doc.pdf.absolute_bottom_margin+6*@rowH, n*@columnW, @doc.pdf.absolute_bottom_margin).stroke
      end
      
      # Set first day of the week to Monday & figure out the start-position for current month
      @order = ['Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday', 'Sunday']
      @startSquare = @order.index(date.strftime("%A"))-1
    end

    def addImage(file, date)
      squareNum = @startSquare + Time.parse(date).day.to_i
      row = squareNum / 7
      col = squareNum % 7
    
      # Calculate the x, y offsets
      x = @doc.pdf.absolute_left_margin + col * @columnW
      y = (@doc.pdf.absolute_top_margin.to_i - @topRow - @rowH * (row+1)) - 3
      
      # Read the source image file and it's corresponding dimensions
      imageMagick = Magick::Image.read(file).first
      imgWidth, imgHeight = imageMagick.columns, imageMagick.rows
      imgRatio = imgWidth.to_f / imgHeight.to_f
      
      # scale to fit in calendar, preserve aspect ratio
      imgWidth, imgHeight = @columnW, (@columnW / imgRatio).to_i if ((imgWidth > imgHeight) and (imgWidth > @columnW))
      imgHeight, imgWidth = @rowH, (@rowH * imgRatio).to_i if ((imgHeight > imgWidth) and (imgHeight > @rowH))
      imgHeight, imgWidth = @rowH, @columnW if (imgHeight == imgWidth)
      
      # center the photo in the calendar square
      x = x + (@columnW-imgWidth)/2 if imgWidth < @columnW
      y = y + (@rowH-imgHeight)/2 if imgHeight < @rowH
      
      @doc.addImage(file, x, y, imgWidth, imgHeight)
    end
  end
end